!
!  Receptors creates Num receptors on rings (polar)
!
!  ISCST3 and AERMOD compute concentration values 
!
!  This program reads in the output file from ISCST3 and AERMOD
!  1) Computes Sy
!  2) Computes Cy
!  3) Selects Cmax
!
      character*8  DTin
      character*40 filein
      character*80 card
      dimension conc(300),AZ(300)
      
      data Sqrt2pi/2.50663/
      
      open(20,file='AZRingOut.txt')
      write(20,1000) 
1000  format('                Distance   Cmax/Q       Sy       Sy      CY1/Q   CmaxT/CmaxO',/,&
            &'   YYMMDDHH        (m) (s*m^3x10^-6)   (deg)     (m) (sx10^-6/m^2)  (none)')      
      
5     write(*,*) ' Name of input file from ISCST3 run: '
      read(*,*)  filein
      open(10,file=filein,status='old',err=5)
      
      write(*,*) ' What was the modeled emission rate (g/s): '
      read(*,*) Qin
      
      write(*,*) ' Number of receptors on each ring: '
      read(*,*) Nrecp
      
      !read past input file headers
      do i=1,8
      read(10,'(a80)') card
         if(i.eq.1) then
         !AERMOD or ISC?
            if(card(3:8).eq.'ISCST3') then
            model = 0
            else
            model = 1
            end if
         end if
      end do
      Nexp = 1  
      
10    continue  
      Cmax = 0.0
       
      do i=1,Nrecp
         if(model.eq.0) then
         read(10,1010,end=500) x,y,conc(i),DTin
1010     format(3f14.0,29x,a8)
         else
         read(10,1015,end=500) x,y,conc(i),DTin
1015     format( 3(1X,F13.5),t90,a8 )        
         end if
         
         if( conc(i).gt.Cmax ) Cmax = Conc(i)
         dist = sqrt(x*x + y*y)
         call AZcalc(x,y,Azout)
         AZ(i) = Azout
      end do
      
      call SyCalc(Nrecp,Conc,AZ,Dist,Symeters,SyDegrees)
      
      key = 0     !I assume we have a zero concentration on either end of the arc.
      call CyCalc(Nrecp,Dist,Conc,AZ,CY1,key)
      
      key = 1     !I make no assumption of whether we have a zero concentraion on
                  !either end. This only makes a difference, when the last 
                  !concentration along the arc is nonzero.
      call CyCalc(Nrecp,Dist,Conc,AZ,CY2,key)
      
      CmaxTheory = CY1/(Symeters*Sqrt2pi)
      if(Cmax.gt.0) then
      Ratio = CmaxTheory/Cmax
      else
      Ratio = -999.9
      end if
      
      ! ISCST3 and AERMOD computed concentration values are expressed as micrograms/m^3
      ! For Kincaid, I set Q = 1000.0, so to retain concentrations in micrograms, I need
      ! to divide here by 1000. Eventually, I called for Qin to be provided.
      write(20,2000) DTin,Dist,Cmax/Qin,Sydegrees,Symeters,CY1/Qin,Ratio
2000  format(3x,a8,3x,f10.3,1pe10.3,2(0pf10.3),1pe10.3,0pf10.4)
      
      Nexp = Nexp + 1
      go to 10
500   continue      
      
      stop
      end
 
      SUBROUTINE CyCalc(NUM,Dist,C,Y,CYTRAP,key)

      PARAMETER (NMAX=300)
      
      DATA PI/3.14159/
!
!     AREA CALCULATES THE INTEGRAL OF C OVER WY BY
!     TRAPIZOIDAL INTEGRATION. IT IS NOT NECESSARY
!     THAT VALUES OF Y BE EVENLY SPACED.
!
!     NUM=NUMBER OF DATA POINTS TO BE INTEGRATED
!     C concentration divided by q
!     WY = distance from center of mass along arc
!          having radius equal to average distance of
!          receptors from the source (m)
!     CYTRAP=INTEGRAL OF C OVER WY

!     key = 0 I assume the arc of concentrations is bounded by zeros.
!             So I add little triangles to each end of the integral.

!     key ne. 0, I do not add the little triangles meantioned above.

!     This is an old program used in many ways over the years. 
!     In this application, we are analyzing modeled concentrations, so
!     we have no missing values (values < 0). If iTest = 0 we do not fill
!     in missing values; I have hardwired iTest to be zero here.
!     However, there may be some cases in the future where I will use this
!     subroutine, and will want to fill in for missing values, so I have
!     left iTest and Function cFill in here, just in case.

      DIMENSION C(NMAX),Y(NMAX),wr(nmax),wc(nmax)

!     test number of values
      if( num.gt.nmax ) then
	  write(*,*) 'Subroutine areatp '
	  write(*,*) 'Num = ',num,' nmax = ',nmax
	  stop
	  end if


      itest  = 1
      icount = 0
!     COPY DATA INTO WORK ARRAYS
      DO 10 I=1,NUM
	  if( c(i) .ge. 0 .and. itest.eq.1 ) then
	  icount = icount + 1
	  WR(I) = Y(I)
      WC(I) = C(I)
	  else
	  WR(I) = Y(I)
	     if(c(i).ge.0 ) then
	     WC(i) = c(i)
	     else
         WC(i) = cfill(num,i,c)
         end if
	  end if
	  
	  if( itest.eq. 1) then
	  nwork = icount
	  else
	  nwork = num
	  end if
10    CONTINUE

!     SORT DATA IN TERMS OF R, CARRY C-VALUES ALONG
!     NOTE:  SORTED IN TERMS OF R, FROM SMALLEST TO LARGEST
      CALL SORT2(WR,WC,NUM,IERR)


!     NOW READY TO INTEGRATE OVER WR

      IF( NUM.LE.2 ) THEN
      CYTRAP = -999.
      GO TO 100
      END IF

      cytrap = 0
      DO 50 I = 2,NUM
!     NOTE: WR IS SORTED SMALLEST TO LARGEST
      c2 = wc(i)
	  c1 = wc(i-1)
  	  r2 = wr(i)
	  r1 = wr(i-1)
	  cavg = 0.5*(c1+c2)
	  delr = abs(r1-r2 )
	  ctest = cavg*delr
      cytrap = cytrap + ctest
50    CONTINUE
!
!     assume first and last increment are bounded by
!     zero.
!     if (key) is not zero, then
!     add small 'tails' to match this assumption
!
      if(key.ne.0) then
	  delr = abs(wr(2)-wr(1))
	  f1 = 0.5*wc(1)*delr

	  delr = abs(wr(num)-wr(num-1))
	  f2 = 0.5*wc(num)*delr

	  else
	  f1 = 0.0
	  f2 = 0.0
	  end if

      cytrap = cytrap + f1 + f2
      
      ! I expressed the position of the receptors along the arc
      ! by their azimuth values relative to North.
      
      ! To convert Cy(deg^-1) to Cy(m^-1), Basis: Circumfrance = 2*pi*radius 
      cytrap = pi*dist*cytrap/180.0
      

100   continue
      RETURN
      END
      
      function cfill(num,index,c)
!
!     try to interpolate for a missing value
!     within the concentration pattern
!
      parameter (nmax=300)
      dimension c(nmax),y(nmax)
!
      if(num.gt.nmax) then
      write(*,*)'In function cfill, and num>nmax'
      stop
      end if

!     find valid conc. below index position
      if( index.eq.1 ) then
      clow = 0.0
      go to 20
      end if

      ilook = index -1
      if( c(ilook).ge.0.0 ) then
      clow = c(ilook)
      else
      clow = 0.0
      end if

20    continue
!     find valid conc. above index position
      if( index.eq.num ) then
      chigh = 0.0
      go to 30
      end if

      ilook = index + 1
      if( c(ilook).ge.0.0 ) then
      chigh = c(ilook)
      else
      chigh = 0.0
      end if

30    continue
      cfill = 0.5*(clow+chigh)
      return
      end
      
      SUBROUTINE SORT2(Y,X,N,IERR)

      PARAMETER (NMAX=300)
!
!   SHELL SORT OF N VALUES IN Y-ARRAY FROM
!   LARGEST TO SMALLEST.
!
      INTEGER I,J,J1,GAP,NMG,IERR
      REAL TY,TX
      REAL Y(NMAX),X(NMAX)
!
      IF(N.GT.NMAX) THEN
      IERR = -4
      GO TO 999
      END IF
! 
      IF(N.GE.1) GO TO 10
      IERR = -5
      GO TO 999
!
   10 IF(N.EQ.1) GO TO 999
!
!  ONE ELEMENT ARRAYS ARE SORTED
!  BY DEFINITION.
!
      GAP = N
   20 GAP = GAP/2
      NMG = N - GAP
      DO 40 J1 = 1,NMG
      I = J1 + GAP
!
!   LOOP J = J1,1,-GAP
!
      J = J1
   30 IF(Y(J).LE.Y(I)) GO TO 40
!
!  SWAP OUT-OF-ORDER PAIRS
!
!       SET IERR TO +2 TO INDICATE
!       REORDERING WAS NEEDED TO
!       SORT ARRAY
!
      IERR = +2
!
      TY = Y(I)
      TX = X(I)
!
      Y(I) = Y(J)
      X(I) = X(J)
!
      Y(J) = TY
      X(J) = TX
!
!   KEEP OLD POINTER FOR NEXT TIME THROUGH
!
      I = J
      J = J - GAP
      IF(J.GE.1) GO TO 30
   40 CONTINUE
      IF(GAP.GT.1) GO TO 20
  999 RETURN
      END

      
      subroutine AZcalc(x,y,Azout)
      ! compute receptor azimuth 
      convert = 3.14159/180.0
      
      angle = atan2(y,x)
      Azout = angle/convert
      
      return
      end
      
      subroutine SyCalc(Num,Conc,AZ,Dist,Symeters,Sydegrees)
      !Compute lateral dispersion for this arc
      dimension Conc(300),AZ(300)
      
      data NumCL/300/,pi/3.14159/
      
      if(Num .gt. NumCL ) then
      write(*,*) 'Need to increase value of NumCL'
      write(*,*) 'Has more than: ',NumCL,' values.'
      stop
      end if

      SumC   = 0.0
      SumCyy = 0.0
      iNum   = 0
      
      do ival=1,Num
       
            Y = AZ(ival)
            C = Conc(ival)
       
            if( C .ge. 0.0 ) then
               iNum   = iNum + 1
               Sumc   = Sumc + C
               Sumcyy = Sumcyy + C * Y * Y
            end if
      end do         ! ival

!     We have all the observed data; compute Sy
      if( Sumc .gt. 0.0 ) then
         Sydegrees = sqrt( Sumcyy/Sumc )
         Symeters  = pi*Dist*Sydegrees/180.0
      else
         Sydegrees = -99.0
         Symeters  = 0.0
      end if

      
      return
      end
      