!
!    I have created a file of CY, SY and Cmax values from observations
!    I have created a file of CY, Sy and Cmax values from AERMOD estimates.
!    I have created a file of Cy, Sy and Cmax values from ISCST3 estimates.
!
!    There are a few cases where I have results from OBS but insufficient
!    meteorology to run AERMOD.
!
!    There are cases when ISCST2 predicted zero concentratoin at the surface
!    because it thought the plume was above the mixing height.
!
!    This program reads the AERMOD-results, and searches for a match
!    in the OBS-results and in the ISCST3-results. When all three are found, 
!    I list out on one line the combined results.
!
!
!    To keep things simple, I have hardwired the filenames. But the program could be
!    easily updated to accept model filenames as input.

      character*8   AermodDate,ObsDate,ISCDate
      character*80  aermodresults,iscresults
      character*180 obsanalysis
      
      dimension     OBK(2,12,31,24)
      
      ISCZeros = 0
      Iaerobs  = 0
      Iall     = 0
      
      Open(40,file='KIN-Combined-NoFill-Results.txt')
      WRITE(40,1050)
1050  format('                       DUR     Dist         Arc          Q           OBK            OMAX    &
      &       CMAX             CY           CMAX/Q          CY/Q          Sy          PHIC                     Distance &
      &  Cmax/Q       Sy       Sy      CY1/Q   CmaxT/CmaxO                    Distance   Cmax/Q       Sy  &
      &     Sy      CY1/Q   CmaxT/CmaxO')
      write(40,1060)
1060  format('     Exp  YYMMDDHHMM  (min)    (m)         (km)        (g/s)         (m)        (gx10^-6/m^3)  &
      &(gx10^-6/m^3)  (sx10^-6/m^2)  (sx10^-6/m^3)  (sx10^-6/m^2)      (m)        (degrees)    YYMMDDHH        (m)&
      & (s*m^3x10^-6)   (deg)     (m) (sx10^-6/m^2)  (none)         YYMMDDHH        (m) (s*m^3x10^-6)   (deg) &
      &    (m) (sx10^-6/m^2)  (none)')      
            
      open(10,file='KincaidOBSAnalysisNoFill.txt',status='old')
      open(15,file='KIN-iscst3-Results.txt',status='old')
      
      open(25,file='Kincutsurf-all.txt',status='old')
      !Read in AERMOD OBK values to combine with the combined results.
      !Read past header
      Read(25,*) 
2     continue
      read(25,1065,end=5) iyr,imonth,iday,jday,ihr,OBKin
1065  format(i2,2i3,i4,i3,t57,f8.1)   
      iyear = iyr-79
      OBK(iyear,imonth,iday,ihr) = OBKin   
      go to 2
5     continue
      close (25)      
        
      open(20,file='KIN-AERMOD-Results.txt',status='old')
      !read past headers
      read(20,*)
      read(20,*)
      
10    continue
      rewind (10)
      read(10,*)
      read(10,*)
      
      !read in AERMOD result
      read(20,1000,end=900) aermodresults
1000  format(a80)
      !extract date/time and arc info.
      read(aermodresults,1005) AermodDate,Distance
1005  format(3x,a8,4x,f9.3)
      !convert distance to km
      Distance = Distance/1000.0
      idist = Distance
      
20    continue
      !search obs-results for a match
      read(10,2000,end=10) obsanalysis
2000  format(a180)      
      !extract data and arc info.
      read(obsanalysis,2005) ObsDate,Arc
2005  format(t11,a8,t43,f5.2)
      iArc = Arc
      
      !if dates and distances match list combined results
      !then go up to 10 to fetch the next modeling results
      if( AermodDate.eq.ObsDate .and. idist.eq.iArc ) then
      iaerobs = iaerobs + 1     !We have found a match between AERMOD and OBS
          rewind (15)
          !read past headers
          read(15,*)
          read(15,*)
          !now search the ISC results and see if we can find a match
30        continue
          read(15,1000,end=10) iscresults
          read(iscresults,1005) ISCDate,distISC
          !convert distISC to km
          idist = distISC/1000.0
          if(ISCDate.eq.ObsDate .and. idist.eq.iArc) then
          Iall = Iall + 1  !we have a match for all three
          !we have results for all three OBS, AERMOD, ISCST3
          !list out results
             !there are cases in the ISC results where no concentrations
             !were computed because the plume was estimated to be above the
             !mixing height. We see these with weird results CY<0. I catch
             !these here prior to listing.
             read(iscresults,2900) CY
2900         format(t55,g10.0)

                if(CY.le.0.0 ) ISCZeros = ISCZeros + 1
                
                if( CY.gt.0.0 ) then    

                !read obsanalysis so we can insert Aermod's OBK
                read(obsanalysis,1020) itest,iyr,imonth,iday,ihour,imin,idur,Rbar,arcs,Q,OBKin,&
                     &OMAX,CMAX,CDY,CMAXoQ,CDYoQ,SIGY,PHIC
1020            format(3x,i5,2x,5i2.2,2x,i2,4(f10.2,3x),5(3x,1pe12.4),2x,0pf12.3,f12.2) 
                iyear = iyr-79
                OBKout = OBK(iyear,imonth,iday,ihour)
                write(obsanalysis,1020) itest,iyear,imonth,iday,ihour,imin,idur,Rbar,arcs,Q,OBKout,&
                     &OMAX,CMAX,CDY,CMAXoQ,CDYoQ,SIGY,PHIC
                                     
                write(40,3000) obsanalysis,aermodresults,iscresults
3000            format(a180,2a80)
                end if
          go to 10
          else
          go to 30
          end if
      end if
      
      !no match, fetch new obs-results
      go to 20
      
900   continue

      !I added these counters so that I would know how many arc-hours were lost
      !because ISCST3 had zero concentrations at the surface.
      write(40,*) '    ISC Zero Cy: ',ISCZeros
      write(40,*) '    Iaerobs:     ',Iaerobs
      write(40,*) '    iallthree:   ',Iall
      stop
      end 
                        
      
      